function _defineProperty(obj, key, value) {
    if (key in obj) {
        Object.defineProperty(obj, key, {
            value: value,
            enumerable: true,
            configurable: true,
            writable: true
        });
    } else {
        obj[key] = value;
    }
    return obj;
}
function _objectSpread(target) {
    for(var i = 1; i < arguments.length; i++){
        var source = arguments[i] != null ? arguments[i] : {};
        var ownKeys = Object.keys(source);
        if (typeof Object.getOwnPropertySymbols === "function") {
            ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function(sym) {
                return Object.getOwnPropertyDescriptor(source, sym).enumerable;
            }));
        }
        ownKeys.forEach(function(key) {
            _defineProperty(target, key, source[key]);
        });
    }
    return target;
}
import * as Joi from 'joi';
import path from 'path';
import { parseOptions, optionsSchema, preSpawnSchema, spawnSchema } from './options';
var JQ_PATH = process.env.JQ_PATH || path.join(__dirname, '..', 'bin', 'jq');
export var FILTER_UNDEFINED_ERROR = 'node-jq: invalid filter argument supplied: "undefined"';
export var INPUT_JSON_UNDEFINED_ERROR = 'node-jq: invalid json argument supplied: "undefined"';
export var INPUT_STRING_ERROR = 'node-jq: invalid json string argument supplied';
var NODE_JQ_ERROR_TEMPLATE = 'node-jq: invalid {#label} ' + 'argument supplied{if(#label == "path" && #type == "json", " (not a .json file)", "")}' + '{if(#label == "path" && #type == "path", " (not a valid path)", "")}: ' + '"{if(#value != undefined, #value, "undefined")}"';
var messages = {
    'any.invalid': NODE_JQ_ERROR_TEMPLATE,
    'any.required': NODE_JQ_ERROR_TEMPLATE,
    'string.base': NODE_JQ_ERROR_TEMPLATE,
    'string.empty': NODE_JQ_ERROR_TEMPLATE
};
var validateArguments = function(filter, json, options) {
    var context = {
        filter: filter,
        json: json
    };
    var validatedOptions = Joi.attempt(options, optionsSchema);
    var validatedPreSpawn = Joi.attempt(context, preSpawnSchema.tailor(validatedOptions.input), {
        messages: messages,
        errors: {
            wrap: {
                label: ''
            }
        }
    });
    var validatedArgs = parseOptions(validatedOptions, validatedPreSpawn.filter, validatedPreSpawn.json);
    var validatedSpawn = Joi.attempt({}, spawnSchema.tailor(validatedOptions.input), {
        context: _objectSpread({}, validatedPreSpawn, {
            options: validatedOptions
        })
    });
    if (validatedOptions.input === 'file') {
        return {
            args: validatedArgs,
            stdin: validatedSpawn.stdin
        };
    }
    return {
        args: validatedArgs,
        stdin: validatedSpawn.stdin
    };
};
export var commandFactory = function(filter, json) {
    var options = arguments.length > 2 && arguments[2] !== void 0 ? arguments[2] : {}, jqPath = arguments.length > 3 ? arguments[3] : void 0;
    var command = jqPath ? path.join(jqPath, './jq') : JQ_PATH;
    var result = validateArguments(filter, json, options);
    return {
        command: command,
        args: result.args,
        stdin: result.stdin
    };
};
